@@ -3,9 +3,9 @@
  *
  * Implementation of the Domain-Free Mandatory Access Control.
  *
- * Copyright (C) 2005-2007  NTT DATA CORPORATION
+ * Copyright (C) 2005-2008  NTT DATA CORPORATION
  *
- * Version: 1.5.0   2007/09/20
+ * Version: 1.5.4   2008/05/10
  *
  * This file is applicable to both 2.4.30 and 2.6.11 and later.
  * See README.ccs for ChangeLog.
@@ -35,9 +35,11 @@ static int AddReservedEntry(const u16 min_port, const u16 max_port, const int is
 	struct reserved_entry *new_entry, *ptr;
 	static DECLARE_MUTEX(lock);
 	int error = -ENOMEM;
+	down(&lock);
 	for (ptr = reservedport_list; ptr; ptr = ptr->next) {
 		if (ptr->min_port == min_port && max_port == ptr->max_port) {
 			ptr->is_deleted = is_delete;
+			error = 0;
 			goto out;
 		}
 	}
@@ -48,7 +50,7 @@ static int AddReservedEntry(const u16 min_port, const u16 max_port, const int is
 	if ((new_entry = alloc_element(sizeof(*new_entry))) == NULL) goto out;
 	new_entry->min_port = min_port;
 	new_entry->max_port = max_port;
-	mb(); /* Instead of using spinlock. */
+	mb(); /* Avoid out-of-order execution. */
 	if ((ptr = reservedport_list) != NULL) {
 		while (ptr->next) ptr = ptr->next; ptr->next = new_entry;
 	} else {
