@@ -3,9 +3,9 @@
  *
  * Implementation of the Domain-Free Mandatory Access Control.
  *
- * Copyright (C) 2005-2007  NTT DATA CORPORATION
+ * Copyright (C) 2005-2008  NTT DATA CORPORATION
  *
- * Version: 1.4.3   2007/11/11
+ * Version: 1.5.4   2008/05/10
  *
  * This file is applicable to both 2.4.30 and 2.6.11 and later.
  * See README.ccs for ChangeLog.
@@ -16,13 +16,13 @@
 #include <linux/ccs_common.h>
 #include <linux/sakura.h>
 #include <linux/realpath.h>
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,20)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 20)
 #include <linux/mount.h>
 #include <linux/mnt_namespace.h>
 #else
 #include <linux/namespace.h>
 #endif
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,0)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 5, 0)
 #include <linux/namei.h>
 #endif
 
@@ -31,9 +31,9 @@
 int SAKURA_MayMount(struct nameidata *nd)
 {
 	int flag = 0;
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,19)
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 19)
 	struct namespace *namespace = current->namespace;
-#elif LINUX_VERSION_CODE < KERNEL_VERSION(2,6,20)
+#elif LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 20)
 	struct namespace *namespace = current->nsproxy->namespace;
 #else
 	struct mnt_namespace *namespace = current->nsproxy->mnt_ns;
@@ -47,23 +47,30 @@ int SAKURA_MayMount(struct nameidata *nd)
 			spin_lock(&dcache_lock);
 			if (IS_ROOT(dentry) || !d_unhashed(dentry)) {
 				while (1) {
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 25)
+					if (nd->path.mnt->mnt_root == vfsmnt->mnt_root && nd->path.dentry == dentry) {
+						flag = 1;
+						break;
+					}
+#else
 					if (nd->mnt->mnt_root == vfsmnt->mnt_root && nd->dentry == dentry) {
 						flag = 1;
 						break;
 					}
+#endif
 					if (dentry == vfsmnt->mnt_root || IS_ROOT(dentry)) {
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,0)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 5, 0)
 						spin_lock(&vfsmount_lock);
 #endif
 						if (vfsmnt->mnt_parent == vfsmnt) {
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,0)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 5, 0)
 							spin_unlock(&vfsmount_lock);
 #endif
 							break;
 						}
 						dentry = vfsmnt->mnt_mountpoint;
 						vfsmnt = vfsmnt->mnt_parent;
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,0)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 5, 0)
 						spin_unlock(&vfsmount_lock);
 #endif
 						continue;
@@ -78,7 +85,11 @@ int SAKURA_MayMount(struct nameidata *nd)
 	if (flag) {
 		int error = -EPERM;
 		const int is_enforce = CheckCCSEnforce(CCS_SAKURA_DENY_CONCEAL_MOUNT);
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 25)
+		const char *dir = realpath_from_dentry(nd->path.dentry, nd->path.mnt);
+#else
 		const char *dir = realpath_from_dentry(nd->dentry, nd->mnt);
+#endif
 		if (dir) {
 			const char *exename = GetEXE();
 			printk("SAKURA-%s: mount %s (pid=%d:exe=%s): Permission denied.\n", GetMSG(is_enforce), dir, current->pid, exename);
@@ -90,6 +101,5 @@ int SAKURA_MayMount(struct nameidata *nd)
 	}
 	return 0;
 }
-EXPORT_SYMBOL(SAKURA_MayMount);
 
 /***** SAKURA Linux end. *****/
