@@ -3,9 +3,9 @@
  *
  * Implementation of the Domain-Free Mandatory Access Control.
  *
- * Copyright (C) 2005-2007  NTT DATA CORPORATION
+ * Copyright (C) 2005-2008  NTT DATA CORPORATION
  *
- * Version: 1.5.2   2007/12/05
+ * Version: 1.5.4   2008/05/10
  *
  * This file is applicable to both 2.4.30 and 2.6.11 and later.
  * See README.ccs for ChangeLog.
@@ -16,7 +16,7 @@
 #include <linux/ccs_common.h>
 #include <linux/sakura.h>
 #include <linux/realpath.h>
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,0)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 5, 0)
 #include <linux/namei.h>
 #else
 #include <linux/fs.h>
@@ -60,7 +60,7 @@ static int AddPivotRootACL(const char *old_root, const char *new_root, const int
 	if ((new_entry = alloc_element(sizeof(*new_entry))) == NULL) goto out;
 	new_entry->old_root = saved_old_root;
 	new_entry->new_root = saved_new_root;
-	mb(); /* Instead of using spinlock. */
+	mb(); /* Avoid out-of-order execution. */
 	if ((ptr = pivot_root_list) != NULL) {
 		while (ptr->next) ptr = ptr->next; ptr->next = new_entry;
 	} else {
@@ -78,8 +78,13 @@ int CheckPivotRootPermission(struct nameidata *old_nd, struct nameidata *new_nd)
 	int error = -EPERM;
 	char *old_root, *new_root;
 	if (!CheckCCSFlags(CCS_SAKURA_RESTRICT_PIVOT_ROOT)) return 0;
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 25)
+	old_root = realpath_from_dentry(old_nd->path.dentry, old_nd->path.mnt);
+	new_root = realpath_from_dentry(new_nd->path.dentry, new_nd->path.mnt);
+#else
 	old_root = realpath_from_dentry(old_nd->dentry, old_nd->mnt);
 	new_root = realpath_from_dentry(new_nd->dentry, new_nd->mnt);
+#endif
 	if (old_root && new_root) {
 		struct path_info old_root_dir, new_root_dir;
 		old_root_dir.name = old_root;
@@ -113,7 +118,6 @@ int CheckPivotRootPermission(struct nameidata *old_nd, struct nameidata *new_nd)
 	ccs_free(new_root);
 	return error;
 }
-EXPORT_SYMBOL(CheckPivotRootPermission);
 
 int AddPivotRootPolicy(char *data, const int is_delete)
 {
