@@ -3,9 +3,9 @@
  *
  * Implementation of the Domain-Based Mandatory Access Control.
  *
- * Copyright (C) 2005-2006  NTT DATA CORPORATION
+ * Copyright (C) 2005-2008  NTT DATA CORPORATION
  *
- * Version: 1.3.1   2006/12/08
+ * Version: 1.5.4   2008/05/10
  *
  * This file is applicable to both 2.4.30 and 2.6.11 and later.
  * See README.ccs for ChangeLog.
@@ -16,6 +16,10 @@
 #include <linux/ccs_common.h>
 #include <linux/tomoyo.h>
 #include <linux/realpath.h>
+#include <linux/version.h>
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 24)
+#define find_task_by_pid find_task_by_vpid
+#endif
 
 /*************************  VARIABLES  *************************/
 
@@ -26,38 +30,36 @@ extern struct semaphore domain_acl_lock;
 
 /*************************  AUDIT FUNCTIONS  *************************/
 
-#ifdef CONFIG_TOMOYO_AUDIT
-static int AuditSignalLog(const int signal, const char *dest_pattern, const int is_granted)
+static int AuditSignalLog(const int signal, const struct path_info *dest_domain, const int is_granted)
 {
 	char *buf;
 	int len;
 	if (CanSaveAuditLog(is_granted) < 0) return -ENOMEM;
-	len = strlen(dest_pattern);
+	len = dest_domain->total_len;
 	if ((buf = InitAuditLog(&len)) == NULL) return -ENOMEM;
-	snprintf(buf + strlen(buf), len - strlen(buf) - 1, KEYWORD_ALLOW_SIGNAL "%d %s\n", signal, dest_pattern);
+	snprintf(buf + strlen(buf), len - strlen(buf) - 1, KEYWORD_ALLOW_SIGNAL "%d %s\n", signal, dest_domain->name);
 	return WriteAuditLog(buf, is_granted);
 }
-#else
-static inline void AuditSignalLog(const int signal, const char *dest_pattern, const int is_granted) {}
-#endif
 
 /*************************  SIGNAL ACL HANDLER  *************************/
 
-static int AddSignalEntry(const int sig, const char *dest_pattern, struct domain_info *domain, const int is_delete, const struct condition_list *condition)
+static int AddSignalEntry(const int sig, const char *dest_pattern, struct domain_info *domain, const struct condition_list *condition, const u8 is_delete)
 {
 	struct acl_info *ptr;
-	const char *saved_dest_pattern;
-	const unsigned int type_hash = MAKE_ACL_TYPE(TYPE_SIGNAL_ACL) + MAKE_ACL_HASH(sig);
+	const struct path_info *saved_dest_pattern;
+	const u16 hash = sig;
 	int error = -ENOMEM;
 	if (!domain) return -EINVAL;
-	if (!dest_pattern || !IsCorrectDomain(dest_pattern)) return -EINVAL;
+	if (!dest_pattern || !IsCorrectDomain(dest_pattern, __FUNCTION__)) return -EINVAL;
+	if ((saved_dest_pattern = SaveName(dest_pattern)) == NULL) return -ENOMEM;
 	down(&domain_acl_lock);
 	if (!is_delete) {
 		if ((ptr = domain->first_acl_ptr) == NULL) goto first_entry;
 		while (1) {
-			SIGNAL_ACL_RECORD *new_ptr;
-			if (ptr->type_hash == type_hash && ptr->cond == condition) {
-				if (strcmp(((SIGNAL_ACL_RECORD *) ptr)->domainname, dest_pattern) == 0) {
+			struct signal_acl_record *new_ptr = (struct signal_acl_record *) ptr;
+			if (ptr->type == TYPE_SIGNAL_ACL && new_ptr->sig == hash && ptr->cond == condition) {
+				if (!pathcmp(new_ptr->domainname, saved_dest_pattern)) {
+					ptr->is_deleted = 0;
 					/* Found. Nothing to do. */
 					error = 0;
 					break;
@@ -69,21 +71,21 @@ static int AddSignalEntry(const int sig, const char *dest_pattern, struct domain
 			}
 		first_entry: ;
 			/* Not found. Append it to the tail. */
-			if ((saved_dest_pattern = SaveName(dest_pattern)) == NULL) break;
-			if ((new_ptr = (SIGNAL_ACL_RECORD *) alloc_element(sizeof(SIGNAL_ACL_RECORD))) == NULL) break;
-			new_ptr->head.type_hash = type_hash;
+			if ((new_ptr = alloc_element(sizeof(*new_ptr))) == NULL) break;
+			new_ptr->head.type = TYPE_SIGNAL_ACL;
+			new_ptr->sig = hash;
 			new_ptr->head.cond = condition;
 			new_ptr->domainname = saved_dest_pattern;
 			error = AddDomainACL(ptr, domain, (struct acl_info *) new_ptr);
 			break;
 		}
 	} else {
-		struct acl_info *prev = NULL;
 		error = -ENOENT;
-		for (ptr = domain->first_acl_ptr; ptr; prev = ptr, ptr = ptr->next) {
-			if (type_hash != ptr->type_hash || ptr->cond != condition) continue;
-			if (strcmp(((SIGNAL_ACL_RECORD *) ptr)->domainname, dest_pattern)) continue;
-			error = DelDomainACL(prev, domain, ptr);
+		for (ptr = domain->first_acl_ptr; ptr; ptr = ptr->next) {
+			struct signal_acl_record *ptr2 = (struct signal_acl_record *) ptr;
+			if (ptr->type != TYPE_SIGNAL_ACL || ptr->is_deleted || ptr2->sig != hash || ptr->cond != condition) continue;
+			if (pathcmp(ptr2->domainname, saved_dest_pattern)) continue;
+			error = DelDomainACL(ptr);
 			break;
 		}
 	}
@@ -97,7 +99,7 @@ int CheckSignalACL(const int sig, const int pid)
 	struct domain_info *dest = NULL;
 	const char *dest_pattern;
 	struct acl_info *ptr;
-	const unsigned int type_hash = MAKE_ACL_TYPE(TYPE_SIGNAL_ACL) + MAKE_ACL_HASH(sig);
+	const u16 hash = sig;
 	const int is_enforce = CheckCCSEnforce(CCS_TOMOYO_MAC_FOR_SIGNAL);
 	if (!CheckCCSFlags(CCS_TOMOYO_MAC_FOR_SIGNAL)) return 0;
 	if (!sig) return 0;                               /* No check for NULL signal. */
@@ -120,27 +122,27 @@ int CheckSignalACL(const int sig, const int pid)
 		AuditSignalLog(sig, dest->domainname, 1);
 		return 0;
 	}
-	dest_pattern = dest->domainname;
+	dest_pattern = dest->domainname->name;
 	for (ptr = domain->first_acl_ptr; ptr; ptr = ptr->next) {
-		if (ptr->type_hash == type_hash && CheckCondition(ptr->cond, NULL) == 0) {
-			const int len = strlen(((SIGNAL_ACL_RECORD *) ptr)->domainname);
-			if (strncmp(((SIGNAL_ACL_RECORD *) ptr)->domainname, dest_pattern, len) == 0 && (dest_pattern[len] == ' ' || dest_pattern[len] == '\0')) break;
+		struct signal_acl_record *ptr2 = (struct signal_acl_record *) ptr;
+		if (ptr->type == TYPE_SIGNAL_ACL && ptr->is_deleted == 0 && ptr2->sig == hash && CheckCondition(ptr->cond, NULL) == 0) {
+			const int len = ptr2->domainname->total_len;
+			if (strncmp(ptr2->domainname->name, dest_pattern, len) == 0 && (dest_pattern[len] == ' ' || dest_pattern[len] == '\0')) break;
 		}
 	}
 	if (ptr) {
-		AuditSignalLog(sig, dest_pattern, 1);
+		AuditSignalLog(sig, dest->domainname, 1);
 		return 0;
 	}
 	if (TomoyoVerboseMode()) {
 		printk("TOMOYO-%s: Signal %d to %s denied for %s\n", GetMSG(is_enforce), sig, GetLastName(dest), GetLastName(domain));
 	}
-	AuditSignalLog(sig, dest_pattern, 0);
-	if (is_enforce) return CheckSupervisor("%s\n" KEYWORD_ALLOW_SIGNAL "%d %s\n", domain->domainname, sig, dest_pattern);
-	if (CheckCCSAccept(CCS_TOMOYO_MAC_FOR_SIGNAL)) AddSignalEntry(sig, dest_pattern, domain, 0, NULL);
+	AuditSignalLog(sig, dest->domainname, 0);
+	if (is_enforce) return CheckSupervisor("%s\n" KEYWORD_ALLOW_SIGNAL "%d %s\n", domain->domainname->name, sig, dest_pattern);
+	if (CheckCCSAccept(CCS_TOMOYO_MAC_FOR_SIGNAL, domain)) AddSignalEntry(sig, dest_pattern, domain, NULL, 0);
 	return 0;
 }
 
-
 int AddSignalPolicy(char *data, struct domain_info *domain, const int is_delete)
 {
 	int sig;
@@ -149,11 +151,9 @@ int AddSignalPolicy(char *data, struct domain_info *domain, const int is_delete)
 		const struct condition_list *condition = NULL;
 		const char *cp = FindConditionPart(domainname + 1);
 		if (cp && (condition = FindOrAssignNewCondition(cp)) == NULL) return -EINVAL;
-		return AddSignalEntry(sig, domainname + 1, domain, is_delete, condition);
+		return AddSignalEntry(sig, domainname + 1, domain, condition, is_delete);
 	}
 	return -EINVAL;
 }
 
-EXPORT_SYMBOL(CheckSignalACL);
-
 /***** TOMOYO Linux end. *****/
